@@ -1,38 +1,44 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+
+# Author: Xiaodong Ming
+
 """
 InputHipims
 ===========
 
-Generate input files for a hipims flood model
+The module is based on the following assumptions:
 
-
-Assumptions:
-
-- Input DEM is a regular DEM file
-- its map unit is meter
-- its cellsize is the same in both x and y direction
-- its reference position is on the lower left corner of the southwest cell
-- All the other grid-based input files must be consistent with the DEM file
+    * Input DEM is a regular DEM file
+    * its map unit is meter
+    * its cellsize is the same in both x and y direction
+    * its reference position is on the lower left corner of the southwest cell
+    * All the other grid-based input files must be consistent with the DEM file
 
 To do:
 
-- generate input (including sub-folder mesh and field) and output folders
-- generate mesh file (DEM.txt) and field files
-- divide model domain into small sections if multiple GPU is used
+    * generate input (including sub-folder mesh and field) and output folders
+    * generate mesh file (DEM.txt) and field files
+    * divide model domain into small sections if multiple GPU is used
 
-Structure:
-   class InputHipims
-       - Initialize an object: __init__
-       - set model parameters: 
-           set_boundary_condition, set_parameter, set_rainfall, 
-           set_gauges_position, set_case_folder, set_runtime, set_device_no,
-           add_user_defined_parameter, set_num_of_sections
-
-------------
+-----------------
 
 """
-# Created on Tue Mar 31 16:03:57 2020
+
+# Structure:
+
+#    class InputHipims
+
+#        - Initialize an object: __init__
+
+#        - set model parameters: 
+
+#            set_boundary_condition, set_parameter, set_rainfall, 
+
+#            set_gauges_position, set_case_folder, set_runtime, set_device_no,
+
+#            add_user_defined_parameter, set_num_of_sections
+
 __author__ = "Xiaodong Ming"
 import os
 import shutil
@@ -46,104 +52,34 @@ from .Landcover import Landcover
 from .Summary import Summary
 from .spatial_analysis import sub2map
 from . import indep_functions as indep_f
-from . import rainfall_processing as rp
 #%% definition of class InputHipims
 class InputHipims:
-    """
-    To define input files for a HiPIMS flood model case
+    """To define input files for a HiPIMS flood model case
+
     Read data, process data, write input files, and save data of a model case.
 
-    Properties (public):
-
-        case_folder: (str) the absolute path of the case folder
-
-        data_folders: (dict) paths for data folders(input, output, mesh, field)
-
+    Attributes:
         num_of_sections: (scalar) number of GPUs to run the model
-
         shape: shape of the DEM array
-
         header: (dict) header of the DEM grid
-
         __attributes_default: (dict) default model attribute names and values
-
         attributes: (dict) model attribute names and values
-
         times:  (list/numpy array) of four values reprenting model run time in
-                    seconds: start, end, output interval, backup interval
-
+                seconds: start, end, output interval, backup interval
         device_no: (int) the gpu device id(s) to run model
-
-        param_per_landcover: dict, argument to set grid parameters using Landcover object. Keys are grid parameter names. Value is a dict with three keys: param_value, land_value, default_value. Refer to Landcover for more details.
-        
-    Objects:
-
+        param_per_landcover: dict, argument to set grid parameters using Landcover
+            object. Keys are grid parameter names. Value is a dict with three
+            keys: param_value, land_value, default_value. Refer to Landcover
+            for more details.
         Sections: a list of objects of child-class InputHipimsSub
-
-        Boundary: To create a Boundary object for boundary conditions
-                The object contains outline_boundary, a dataframe of boundary type, extent, source data, code, ect..., and a boundary subscrpits tuple (cell_subs). For multi-GPU case, a boundary subscrpits tuple (cell_subs_l) based on sub-grids will be created for each section
-
-        outline_boundary: (str) 'open'|'rigid', default outline boundary is open and both h and hU are set as zero
-
-        boundary_list: (list of dicts), each dict contain keys (polyPoints, type, h, hU) to define a IO boundary's position, type, and Input-Output (IO) sources timeseries. Keys including:
-
-            1. polyPoints is a numpy array giving X(1st col) and Y(2nd col) coordinates of points to define the position of a boundary. An empty polyPoints means outline boundary.
-
-            2. type: 'open'(flow out flatly), 'rigid'(no outlet), 'fall'(water flow out like a fall)
-
-            3. h: a two-col numpy array. The 1st col is time(s). The 2nd col is water depth(m)
-
-            4. hU: a two-col numpy array. The 1st col is time(s). The 2nd col is discharge(m3/s) or a three-col numpy array, the 2nd col and the 3rd col are velocities(m/s) in x and y
-                direction, respectively.
-                
-        cell_subs: (list of tuple) subsripts of cells for each boundary
-
-        cell_id: (list of vector) valid id of cells for each boundary
-        
+        Boundary: A boundary object for boundary conditions
         DEM: a Raster object to provide DEM data [alias: Raster].
-
         Rainfall: a Rainfall object to provide rainfall data
-            
-        Landcover: a Landcover object to provide landcover data for setting gridded parameters
-                
+        Landcover: a Landcover object to provide landcover data for setting
+            gridded parameters        
         Summary: a Summary object to record model information
-    
-    Properties (Private):
-
-        _valid_cell_subs: (tuple, int numpy) two numpy array indicating rows and cols of valid cells on the DEM array and sorted based on the valid_cell_id, starting from the bottom-left valid cell towards right and up on the DEM array.
-
-        _outline_cell_subs: (tuple, int numpy) two numpy array indicating rows and cols of outline cells on the DEM array and sorted based on the valid_cell_id from small to large
-
-    Methods (public):
-
-        set_parameter: set grid-based parameters
-
-        set_boundary_condition: set boundary condition with a boundary list
-
-        set_rainfall: set rainfall mask and sources
-
-        set_gauges_position: set X-Y coordinates of monitoring  gauges
-
-        write_input_files: write all input files or a specific file
-
-        write_grid_files: write grid-based data files
-
-        write_boundary_conditions: write boundary sources, if flow time series is given, it will be converted to velocities in x and y directions
-
-        write_gauges_position: write coordinates of monitoring gauges
-
-        write_halo_file: write overlayed cell ID for multiple GPU cases
-
-    Classes:
-        InputHipimsSub: child class of InputHiPIMS, provide information of each sub-domain
-
-        Boundary: provide information of boundary conditions
-
-        ModelSummary: record basic information of an object of InputHiPIMS
-
-    ----------------
-    
     """
+
 #%%****************************************************************************
 #***************************initialize an object*******************************
     # default parameters
@@ -156,6 +92,12 @@ class InputHipims:
                           'cumulative_depth':0, 'hydraulic_conductivity':0,
                           'capillary_head':0, 'water_content_diff':0,
                           'gauges_pos':np.array([[0, 0], [1, 1]])}
+    _file_tag_list = ['z', 'h', 'hU', 'precipitation',
+                      'manning', 'sewer_sink',
+                      'cumulative_depth', 'hydraulic_conductivity',
+                      'capillary_head', 'water_content_diff',
+                      'precipitation_mask', 'precipitation_source',
+                      'boundary_condition', 'gauges_pos']
     __grid_files = ['z', 'h', 'hU', 'precipitation_mask',
                   'manning', 'sewer_sink', 'precipitation',
                   'cumulative_depth', 'hydraulic_conductivity',
@@ -165,10 +107,13 @@ class InputHipims:
                            'capillary_head', 'water_content_diff']
     def __init__(self, dem_data=None, num_of_sections=1, case_folder=None,
                  data_path=None):
-        """
-        dem_data: (Raster object) or (str) provides file name of the DEM data
+        """Initialise the InputHipims object
 
-        data_folder: a path contain at least a DEM file named as 'DEM' with a suffix .gz|.asc|.tif. 'landcover' and 'rain_mask' can also be read if these files were given with one of the three suffix
+        Args:
+            dem_data: (Raster object) or (str) provides file name of the DEM data
+            data_folder: a path contain at least a DEM file named as 'DEM' with a
+                suffix .gz|.asc|.tif. 'landcover' and 'rain_mask' can also be read if
+                these files were given with one of the three suffix
         """
         self.attributes = InputHipims.__attributes_default.copy()
         self.num_of_sections = num_of_sections
@@ -205,8 +150,7 @@ class InputHipims:
         self._initialize_summary_obj()# initialize a Model Summary object
         
     def __str__(self):
-        """
-        To show object summary information when it is called in console
+        """To show object summary information when it is called in console
         """
         self.Summary.display()
         time_str = self.birthday.strftime('%Y-%m-%d %H:%M:%S')
@@ -215,19 +159,58 @@ class InputHipims:
 #******************************************************************************
 #%%**********************Get object attributes**************************************
     def get_case_folder(self):
-        """ Return _case_folder
+        """Get the directory of the case.
+
+        Return: 
+            string: The directory of the case
         """
         return self._case_folder
 
     def get_data_folders(self):
-        """ Return _case_folder
+        """Get the directories and subdirectories of the input and output data.
+        
+        Return:
+            string: a dict with four keys ('input', 'output', 'mesh', 'field')
+            providing their absolute paths
         """
         return self._data_folders
+    
+    def get_input_filenames(self):
+        """Get the input file names.
+        
+        Return:
+            list: a list of string giving all input filenames without 
+        """
+        return self._file_tag_list
 
 #%%**********************Setup the object**************************************
     def set_boundary_condition(self, boundary_list=None,
                                outline_boundary='fall'):
         """To create a Boundary object for boundary conditions
+        
+        Args:
+            boundary_list: (list of dicts), each dict contain keys (polyPoints,
+                type, h, hU) to define a IO boundary's position, type, and
+                Input-Output (IO) sources timeseries. Keys including:
+
+                1. polyPoints: a numpy array giving X(1st col) and Y(2nd col)
+                coordinates of points to define the position of a boundary.
+                An empty polyPoints means outline boundary.
+
+                2. type: 'open'(flow out flatly), 'rigid'(no outlet),
+                'fall'(water flow out like a fall)
+
+                3. h: a two-col numpy array. The 1st col is time(s). The 2nd col
+                is water depth(m)
+
+                4. hU: a two-col numpy array. The 1st col is time(s). The 2nd
+                col is discharge(m3/s) or a three-col numpy array, the 2nd
+                col and the 3rd col are velocities(m/s) in x and y
+                direction, respectively.
+
+            outline_boundary: (str) 'open'|'rigid', default outline boundary is
+                open and both h and hU are set as zero
+
         """
         if boundary_list is None and hasattr(self, 'Boundary'):
             boundary_list = self.Boundary.boundary_list
@@ -246,11 +229,11 @@ class InputHipims:
             self.Summary.set_boundary_attr(self.Boundary)
 
     def set_initial_condition(self, parameter_name, parameter_value):
-        """ Set initial condition for h0, hU0x, hU0y
-
-        **parameter_name**: (str) h0, hU0x, hU0y
+        """Set initial condition for h0, hU0x, hU0y
         
-        **parameter_value**: scalar or numpy array with the same size of DEM.
+        Args:
+            parameter_name: (str) h0, hU0x, hU0y
+            parameter_value: scalar or numpy array with the same size of DEM.
         """
         if parameter_name not in ['h0', 'hU0x', 'hU0y']:
             raise ValueError('Parameter is not recognized: '+parameter_name)
@@ -268,19 +251,24 @@ class InputHipims:
     def set_grid_parameter(self, **kwargs):
         """ Set grid parameter with Landcover object as name=value
 
-        kwargs: Keyword Arguments Specified by a Dictionary
+        Args:
+            kwargs: 
+                Keyword Arguments Specified by a Dictionary
+                keyword: name, from grid_parameter_keys
+                value:  
+                    1. scalar, a uniform parameter value
+                    2. array, gridded parameter value with the same size of DEM
+                    3. dict, contain param_value, land_value, default_value=0
 
-        keyword: name, from grid_parameter_keys
+        Example:
+            Using landcover to set up parameters
+            
+            .. code-block:: python
 
-        value:  
-        
-            1. scalar, a uniform parameter value
-
-            2. array, gridded parameter value with the same size of DEM
-
-            3. dict, contain param_value, land_value, default_value=0
-
-        Return: save a parameter dictionary
+                obj_in.set_landcover(landcover)
+                obj_in.set_grid_parameter(manning={'param_value':[0.025, 0.055],
+                                    'land_value':[[0,1], [2,3]], # 0.025 corresponds to landcover type of 0,1
+                                    'default_value':0.035}
         """
         if not hasattr(self, 'param_per_landcover'):
             # save arguments to call Landcover.to_grid_parameter()
@@ -306,10 +294,14 @@ class InputHipims:
 
     def set_rainfall(self, rain_mask=None, rain_source=None):
         """ Set rainfall mask and rainfall source
-        
-        rain_mask: str [filename of a Raster endswith .gz/asc/tif] numpy int array with the same shape with DEM array a Raster object
-        
-        rain_source: numpy array the 1st column is time in seconds, 2nd to the end columns are rainfall rates in m/s. str [filename of a csv file for rainfall source data]
+
+        Args:
+            rain_mask: str [filename of a Raster endswith .gz/asc/tif]
+                    numpy int array with the same shape with DEM array
+                    a Raster object
+            rain_source: str [filename of a csv file for rainfall source data]
+                    numpy array the 1st column is time in seconds, 2nd to
+                    the end columns are rainfall rates in m/s.
         """
         if not hasattr(self, 'Rainfall'):
             self.Rainfall = Rainfall(self.attributes['precipitation_mask'], 
@@ -330,6 +322,9 @@ class InputHipims:
 
     def set_gauges_position(self, gauges_pos=None):
         """Set coordinates of monitoring gauges
+
+        Args:
+            gauges_pos: (numpy array) the 1st column is X coordinates, 2nd column is the Y coordinates
         """
         if gauges_pos is None:
             gauges_pos = self.attributes['gauges_pos']
@@ -356,10 +351,10 @@ class InputHipims:
 
     def set_case_folder(self, new_folder=None, make_dir=False):
         """ Initialize, renew, or create case and data folders
-
-        new_folder: (str) renew case and data folder if it is given
-
-        make_dir: True|False create folders if it is True
+        
+        Args:
+            new_folder: (str) renew case and data folder if it is given
+            make_dir: True|False create folders if it is True
         """
         # to change case_folder
         if new_folder is None:
@@ -378,9 +373,9 @@ class InputHipims:
     def set_runtime(self, runtime=None):
         """set runtime of the model
 
-        runtime: a list of four values representing start, end, output interval
-
-        and backup interval respectively
+        Args:
+            runtime: a list of four values representing start, end, output interval
+            and backup interval respectively
         """
         if runtime is None:
             runtime = [0, 3600, 3600, 3600]
@@ -397,7 +392,8 @@ class InputHipims:
     def set_device_no(self, device_no=None):
         """set device no of the model
 
-        device_no: int or a list of int corresponding to the number of sections 
+        Args:
+            device_no: int or a list of int corresponding to the number of sections 
         """
         if device_no is None:
             device_no = np.arange(self.num_of_sections)
@@ -406,15 +402,18 @@ class InputHipims:
     
     def set_landcover(self, landcover_data):
         """ Set Landcover object with a Raster object or file
+
+        Args:
+            landcover_data: (string or Raster) A Raster file or its file name of land cover data
         """
         self.Landcover = Landcover(landcover_data, self.DEM)
 
     def add_user_defined_parameter(self, param_name, param_value):
         """ Add a grid-based user-defined parameter to the model
 
-        param_name: (str) name the parameter and the input file name as well
-
-        param_value: (scalar) or (numpy arary) with the same size of DEM array
+        Args:
+            param_name: (str) name the parameter and the input file name as well
+            param_value: (scalar) or (numpy arary) with the same size of DEM array
         """
         if param_name not in InputHipims.__grid_files:
             InputHipims.__grid_files.append(param_name)
@@ -424,7 +423,10 @@ class InputHipims:
     def set_num_of_sections(self, num_of_sections):
         """ set the number of divided sections to run a case
 
-        can transfer single-gpu to multi-gpu and the opposite way
+        It can transfer single-gpu to multi-gpu or the opposite way
+
+        Args:
+            num_of_sections: (int) number of domains
         """
         self.num_of_sections = num_of_sections
         self.set_device_no()
@@ -444,6 +446,9 @@ class InputHipims:
 
     def decomposite_domain(self, num_of_sections):
         """ divide a single-gpu case into a multi-gpu case
+
+        Args:
+            num_of_sections: (int) number of domains
         """
         obj_mg = copy.deepcopy(self)
         obj_mg.num_of_sections = num_of_sections
@@ -461,33 +466,31 @@ class InputHipims:
 #************************Write input files*************************************
     def write_input_files(self, file_tag=None):
         """ Write input files
-        To classify the input files and call functions needed to write each input files
 
-        file_tag: str or list of str including
+        To classify the input files and call functions needed to write each
+        input files
+        
+        Args:
+            file_tag: a string or list of string giving the name(s) of input files without suffix
         """
         self._make_data_dirs()
         grid_files = InputHipims.__grid_files
         if file_tag is None or file_tag == 'all': # write all files
-            file_tag_list = ['z', 'h', 'hU', 'precipitation',
-                             'manning', 'sewer_sink',
-                             'cumulative_depth', 'hydraulic_conductivity',
-                             'capillary_head', 'water_content_diff',
-                             'precipitation_mask', 'precipitation_source',
-                             'boundary_condition', 'gauges_pos']
+            write_list = self._file_tag_list
             if self.num_of_sections > 1:
                 self.write_halo_file()
             self.write_mesh_file()
             self.write_runtime_file()
             self.write_device_file()
         elif type(file_tag) is str:
-            file_tag_list = [file_tag]
+            write_list = [file_tag]
         elif type(file_tag) is list:
-            file_tag_list = file_tag
+            write_list = file_tag
         else:
-            print(file_tag_list)
+            print(self._file_tag_list)
             raise ValueError(('file_tag should be a string or a list of string'
                              'from the above list'))
-        for one_file in file_tag_list:
+        for one_file in write_list:
             if one_file in grid_files: # grid-based files
                 self.write_grid_files(one_file)
             elif one_file == 'boundary_condition':
@@ -504,7 +507,8 @@ class InputHipims:
 
         Public version for both single and multiple GPUs
 
-        file_tag: the pure name of a grid-based file
+        Args:
+            file_tag: the pure name of a grid-based file
         """
         self._make_data_dirs()
         grid_files = InputHipims.__grid_files
@@ -556,6 +560,9 @@ class InputHipims:
         """ Write the gauges position file
 
         Public version for both single and multiple GPUs
+
+        Args:
+            gauge_pos: (2-coloumn numpy array) positions of gauges 
         """
         self._make_data_dirs()
         if gauges_pos is not None:
@@ -596,7 +603,8 @@ class InputHipims:
         print('halo.dat created')
 
     def write_mesh_file(self, is_single_gpu=False):
-        """ Write mesh file DEM.txt, compatoble for both single and multiple GPU model
+        """ Write mesh file DEM.txt, compatoble for both single and multiple
+            GPU model
         """
         self._make_data_dirs()
         if is_single_gpu is True or self.num_of_sections == 1:
@@ -639,11 +647,11 @@ class InputHipims:
                     **kwargs):
         """Show domain map of the object
         """
-        obj_dem = copy.copy(self.DEM)
-        if hasattr(self, 'Sections'):
-            for obj_sub in self.Sections:
-                overlayed_subs = obj_sub.overlayed_cell_subs_global
-                obj_dem.array[overlayed_subs] = np.nan            
+        obj_dem = copy.deepcopy(self.DEM)
+#        if hasattr(self, 'Sections'):
+#            for obj_sub in self.Sections:
+#                overlayed_subs = obj_sub.overlayed_cell_subs_global
+#                obj_dem.array[overlayed_subs] = np.nan            
         fig, ax = obj_dem.mapshow(title=title, cax_str='DEM(m)', **kwargs)
         cell_subs = self.Boundary.cell_subs
         legends = []
@@ -665,27 +673,24 @@ class InputHipims:
     def plot_rainfall_map(self, figname=None, method='sum', **kw):
         """plot rainfall map within model domain
         """
-        rain_source = self.attributes['precipitation_source']
-        rain_mask = self.attributes['precipitation_mask']
-        rain_mask = self.DEM.array*0+rain_mask
-        rain_mask_obj = Raster(array=rain_mask, header=self.header)
-        rain_map_obj = rp.get_spatial_map(rain_source, rain_mask_obj, figname,
-                                          method, **kw)
-        return rain_map_obj
+        fig, ax = self.Rainfall.plot_rainfall_map(method='sum', **kw)
+        cell_subs = self._outline_cell_subs
+        rows = cell_subs[0]
+        cols = cell_subs[1]
+        X, Y = sub2map(rows, cols, self.DEM.header)
+        ax.plot(X, Y, '.k')
+        return fig, ax 
 
     def plot_rainfall_curve(self, start_date=None, method='mean', **kw):
         """ Plot time series of average rainfall rate inside the model domain
 
-        start_date: a datetime object to give the initial date and time of rain method: 'mean'|'max','min','mean'method to calculate gridded rainfall over the model domain
+        Args:
+            start_date: a datetime object to give the initial date and time of rain
+            method: 'mean'|'max','min','mean'method to calculate gridded rainfall 
+                over the model domain
         """
-        rain_source = self.attributes['precipitation_source']
-        rain_mask = self.attributes['precipitation_mask']
-        rain_mask = np.array(rain_mask)
-        rain_mask = self.DEM.array*0+rain_mask
-        plot_data = rp.get_time_series(rain_source, rain_mask, start_date, 
-                                    method=method)
-        rp.plot_time_series(plot_data, method=method, **kw)
-        return plot_data
+        fig, ax = self.Rainfall.plot_time_series(method, **kw)
+        return fig, ax
 
 #%%****************************************************************************
 #*************************** Protected methods ********************************
@@ -693,12 +698,6 @@ class InputHipims:
         """ To get valid_cell_subs and outline_cell_subs for the object
 
         To get the subscripts of each valid cell on grid
-
-        Input arguments are for sub Hipims objects
-
-        _valid_cell_subs
-
-        _outline_cell_subs
         """
         if dem_array is None:
             dem_array = self.DEM.array
@@ -716,8 +715,7 @@ class InputHipims:
         self._outline_cell_subs = (sorted_array[:, 1].astype('int32'),
                                    sorted_array[:, 2].astype('int32'))
     def __divide_grid(self):
-        """
-        Divide DEM grid to sub grids
+        """Divide DEM grid to sub grids
 
         Create objects based on sub-class InputHipimsSub
         """
@@ -780,13 +778,15 @@ class InputHipims:
 
     def _get_vector_value(self, attribute_name, is_multi_gpu=True,
                           add_initial_water=True):
-        """ Generate a single vector for values in each grid cell sorted based on cell IDs
-
-        attribute_name: attribute names based on a grid
+        """ Generate a single vector for values in each grid cell sorted based
+                on cell IDs
+        
+        Args:
+            attribute_name: attribute names based on a grid
 
         Return:
-
-            output_vector: a vector of values in global valid grid cells or a list of vectors for each sub domain
+            vector or a list vectors: a vector of values in global valid grid cells
+                            or a list of vectors for each sub domain
         """
         # get grid value
         dem_shape = self.shape
@@ -861,8 +861,7 @@ class InputHipims:
         return output_vector
 
     def _get_boundary_id_code_array(self, file_tag='z'):
-        """
-        To generate a 4-col array of boundary cell id (0) and code (1~3)
+        """To generate a 4-col array of boundary cell id (0) and code (1~3)
         """
         bound_obj = self.Boundary
         output_array_list = []
@@ -906,8 +905,8 @@ class InputHipims:
 
     def _write_grid_files(self, file_tag, is_multi_gpu=True):
         """ Write input files consistent with the DEM grid
-        Private function called by public function write_grid_files
 
+        Private function called by public function write_grid_files
         file_name: includes ['h','hU','precipitation_mask',
                              'manning','sewer_sink',
                              'cumulative_depth', 'hydraulic_conductivity',
@@ -963,14 +962,11 @@ class InputHipims:
 #*************** Private methods only for the parent class ********************
 #------------------------------------------------------------------------------
     def __write_boundary_conditions(self, field_dir, file_tag='both'):
-        """ Write boundary condition source files,if hU is given as flow timeseries, convert flow to hUx and hUy.
-        
+        """ Write boundary condition source files,if hU is given as flow
+        timeseries, convert flow to hUx and hUy.
         Private function to call by public function write_boundary_conditions
-        
         file_tag: 'h', 'hU', 'both'
-        
         h_BC_[N].dat, hU_BC_[N].dat
-        
         if hU is given as flow timeseries, convert flow to hUx and hUy
         """
         obj_boundary = self.Boundary
@@ -1017,11 +1013,8 @@ class InputHipims:
 
     def __write_gauge_pos(self, file_folder):
         """write monitoring gauges
-        
-            gauges_pos.dat
-        
+        gauges_pos.dat
         file_folder: folder to write file
-        
         gauges_pos: 2-col numpy array of X and Y coordinates
         """
         gauges_pos = self.attributes['gauges_pos']
@@ -1035,11 +1028,8 @@ class InputHipims:
 
     def __write_gauge_ind(self, file_folder):
         """write monitoring gauges index for mult-GPU sections
-
-            gauges_ind.dat
-        
+        gauges_ind.dat
         file_folder: folder to write file
-        
         gauges_ind: 1-col numpy array of index values
         """
         gauges_ind = self.attributes['gauges_ind']
@@ -1053,13 +1043,9 @@ class InputHipims:
 
     def __copy_to_all_sections(self, file_names):
         """ Copy files that are the same in each sections
-        
         file_names: (str) files written in the first seciton [0]
-        
         boundary source files: h_BC_[N].dat, hU_BC_[N].dat
-        
         rainfall source files: precipitation_source_all.dat
-        
         gauges position file: gauges_pos.dat
         """
         if type(file_names) is not list:
@@ -1071,9 +1057,7 @@ class InputHipims:
     
     def _setup_by_files(self, data_path):
         """Read files to setup hipims input object
-        
         DEM, landcover, rain_mask, endswith '.gz', '.asc', or '.tif'
-        
         rain_source.csv
         """
         data_path = self.data_path
@@ -1102,21 +1086,15 @@ class InputHipimsSub(InputHipims):
     """object for each section, child class of InputHipims
 
     Attributes:
-
         sectionNO: the serial number of each section
-
-        _valid_cell_subs: (tuple, int) two numpy array indicating rows and cols of valid cells on the local grid
-
-        valid_cell_subsOnGlobal: (tuple, int) two numpy array indicating rows and cols of valid cells on the global grid
-
+        _valid_cell_subs: (tuple, int) two numpy array indicating rows and cols
+            of valid cells on the local grid
+        valid_cell_subsOnGlobal: (tuple, int) two numpy array indicating rows
+            and cols of valid cells on the global grid
         shared_cells_id: 2-row shared Cells id on a local grid
-
         case_folder: input folder of each section
-
-        _outline_cell_subs: (tuple, int) two numpy array indicating rows and cols of valid cells on a local grid
-    
-    ---------
-
+        _outline_cell_subs: (tuple, int) two numpy array indicating rows and 
+            cols of valid cells on a local grid
     """
     section_id = 0
     def __init__(self, dem_array, header, case_folder, num_of_sections):
@@ -1127,9 +1105,64 @@ class InputHipimsSub(InputHipims):
 
 #%%****************************************************************************
 #********************************Static method*********************************
+def load_object(filename):
+    """load object from a dictionary and return as an InputHipims object
+    Args:
+        filename: a string giving the object file name
+    Return: 
+        An object of InputHipims
+    """
+    obj_dict = indep_f.load_object(filename)
+    
+    if 'DEM' in obj_dict:
+        dem_dict = obj_dict['DEM']
+        obj_dem = Raster(array=dem_dict['array'],
+                                header=dem_dict['header'])
+        obj_dict.pop('DEM')
+    else:
+        raise ValueError(filename+' has no key: DEM')
+    obj_in = InputHipims(dem_data=obj_dem,
+                         num_of_sections=obj_dict['num_of_sections'],
+                         case_folder=obj_dict['_case_folder'])
+    
+    if 'Landcover' in obj_dict:
+        ld_dict = obj_dict['Landcover']
+        mask_header = ld_dict['mask_header']
+        mask_dict = ld_dict['mask_dict']
+        array_shape = (mask_header['nrows'], mask_header['ncols'])
+        mask_array = indep_f._dict2grid(mask_dict, array_shape)
+        ras_landcover = Raster(array=mask_array, header=mask_header)
+        obj_in.set_landcover(ras_landcover)
+        obj_dict.pop('Landcover')
+    
+    if 'Rainfall' in obj_dict:
+        rain_dict = obj_dict['Rainfall']
+        mask_header = rain_dict['mask_header']
+        mask_dict = rain_dict['mask_dict']
+        array_shape = (mask_header['nrows'], mask_header['ncols'])
+        mask_array = indep_f._dict2grid(mask_dict, array_shape)
+        rain_mask = Raster(array=mask_array, header=mask_header)
+        rain_source = np.c_[rain_dict['time_s'], rain_dict['rain_rate']]
+        obj_in.set_rainfall(rain_mask, rain_source)
+        obj_dict.pop('Rainfall')
+    
+    bound_dict = obj_dict['Boundary']
+    for key, value in bound_dict.items():
+        obj_in.Boundary.__dict__[key] = value
+    obj_dict.pop('Boundary')
+    
+    summ_dict = obj_dict['Summary']
+    for key, value in summ_dict.items():
+        obj_in.Summary.__dict__[key] = value
+    obj_dict.pop('Summary')
+    
+    for key, value in obj_dict.items():
+        obj_in.__dict__[key] = value
+    
+    return obj_in
 
 def main():
     print('Class to setup input data')
 
 if __name__=='__main__':
-    main()
+    main()
\ No newline at end of file
