@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # Created on Wed Apr  1 14:56:15 2020
+# Author: Xiaoding Ming
 
 """
 
@@ -8,12 +9,12 @@ Demo functions
 ==============
 
 To do:
-    Create the demo
+    * Create the demo
 
 ---------------
 
 """
-import os
+# import os
 import pkg_resources
 import numpy as np
 from .InputHipims import InputHipims
@@ -24,9 +25,11 @@ def demo_input(num_of_sections=1, set_example_inputs=True,
                figname=None, dpi=200, **kwargs):
     """ A demonstration to generate a hipims input object
     
-    set_example_inputs: (True|False) if True, initial condition, boundary condition, rainfall source, and gauge postion will be set to the input object according to sample data.
-    
-    figname: (string) if given, a domain map will saved
+    Args:
+        set_example_inputs: (True|False) if True, initial condition, boundary 
+            condition, rainfall source, and gauge postion will be set to the 
+            input object according to sample data.
+        figname: (string) if given, a domain map will saved
 
     """
     dem_file = pkg_resources.resource_filename(__name__,
@@ -36,15 +39,21 @@ def demo_input(num_of_sections=1, set_example_inputs=True,
         __set_defaul_input(obj_in)
     # show model summary print(obj_in)
     obj_in.Summary.display()
-    fig, ax = obj_in.domain_show(relocate=True, scale_ratio=1000, **kwargs)
-    ax.set_title('The Upper Lee catchment')
+    fig, ax = obj_in.domain_show(**kwargs)
+    ax.set_title('Domain map')
     if figname is not None:
         fig.savefig(figname, dpi=dpi)
     return obj_in
 
 def demo_output(case_folder, num_of_sections=1):
     """ A demonstration to generate a hipims output object
-    a input folder and files must be created before using this function
+
+    Args:
+        case_folder: (string) path to case folder
+        num_of_sections: (int) number of domains
+    
+    Note:
+        a input folder and files must be created before using this function
     """
     obj_out = OutputHipims(case_folder=case_folder,
                            num_of_sections=num_of_sections)
@@ -52,7 +61,9 @@ def demo_output(case_folder, num_of_sections=1):
 
 def demo_raster(figname=None):
     """ A demonstration to read and show raster files
-    figname: the file name to save the figure
+
+    Args:
+        figname: (string) the file name to save the figure
     """
     dem_file = pkg_resources.resource_filename(__name__,
                                              'sample/Example_DEM.asc')
@@ -61,35 +72,41 @@ def demo_raster(figname=None):
     ax.set_title('The Upper Lee catchment DEM (mAOD)')
     return obj_ras
 
-def get_sample_data(return_path=False):
+def get_sample_data():
     """ Get sample data for demonstartion
 
-    Return:
-        obj_ras: a DEM raster object
-        demo_data: a dictionary with boundary_condition, rain_source, and gauges_pos data
+    Returns:
+        A DEM raster object and a dictionary with boundary_condition,
+        rain_source, and gauges_pos data
     """
     dem_file = pkg_resources.resource_filename(__name__,
                                              'sample/DEM.gz')
-    if return_path:
-        sample_path = os.path.dirname(dem_file)
-        return sample_path
-    else:
-        obj_ras = Raster(dem_file)
-        demo_data_file = pkg_resources.resource_filename(__name__,
-                                                 'sample/Example_data.npy')
-        demo_data = np.load(demo_data_file, allow_pickle='TRUE').item()
-        return obj_ras, demo_data
+    demo_data = {
+        'boundary_condition': [
+            {'polyPoints': np.array([[1427, 195],
+                                 [1446, 243]]),
+             'type': 'open',
+             'hU': [[0, 30], [60, 20]]},
+            {'polyPoints': np.array([[58, 1645],
+                                 [72, 1170]]),
+             'type': 'open',
+             'h': [[0, 12.5], [60, 12.5]]}],
+        'rain_source': np.array([[0, 2.77777778e-06],
+                                 [1800, 2.77777778e-05],
+                                 [3600, 0]]),
+        'gauges_pos': np.array([[300, 1600],
+                                [400, 1200]])
+    }
+    return dem_file, demo_data
     
 # =============private functions==================
 def __set_defaul_input(obj_in):
     """Set some default values for an InputHipims object
     """
     # load data for the demo
-    demo_data_file = pkg_resources.resource_filename(__name__,
-                                             'sample/Example_data.npy')
-    demo_data = np.load(demo_data_file, allow_pickle='TRUE').item()
+    _, demo_data = get_sample_data()
     # define initial condition
-    h0 = obj_in.Raster.array+0
+    h0 = obj_in.DEM.array+0
     h0[np.isnan(h0)] = 0
     h0[h0 < 50] = 0
     h0[h0 >= 50] = 1
